@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of totally random functions that didn't fit elsewhere
  *
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
@@ -216,6 +217,13 @@ rb_compare_gtimeval (GTimeVal *a, GTimeVal *b)
 		return -1;
 }
 
+int
+rb_safe_strcmp (const char *a,
+                const char *b)
+{
+	return (!a && !b) ? 0 : (a && !b) || (!a && b) ? 1 : strcmp (a, b);
+}
+
 /* Taken from totem/video-utils.c CVS HEAD 2004-04-22 */
 static void
 totem_pixbuf_mirror (GdkPixbuf *pixbuf)
@@ -404,15 +412,29 @@ purge_useless_threads (gpointer data)
 	return TRUE;
 }
 
+
+static GStaticRecMutex rb_gdk_mutex;
+
+static void
+_threads_enter (void)
+{
+	g_static_rec_mutex_lock (&rb_gdk_mutex);
+}
+
+static void
+_threads_leave (void)
+{
+	g_static_rec_mutex_unlock (&rb_gdk_mutex);
+}
+
 void
 rb_threads_init (void)
 {
 	private_is_primary_thread = g_private_new (NULL);
 	g_private_set (private_is_primary_thread, GUINT_TO_POINTER (1));
 
-	/* not really necessary, but in case it does something besides
-	 * set up lock functions some day..
-	 */
+	g_static_rec_mutex_init (&rb_gdk_mutex);
+	gdk_threads_set_lock_functions (_threads_enter, _threads_leave);
 	gdk_threads_init ();
 
 	/* purge useless thread-pool threads occasionally */
@@ -443,7 +465,7 @@ rb_string_split_words (const gchar *string)
 	while (*cur_read) {
 		switch (g_unichar_type (*cur_read)) {
 		case G_UNICODE_UNASSIGNED:
-			g_warning ("unassigned unicode character type found");
+			rb_debug ("unassigned unicode character type found");
 			/* fall through */
 		case G_UNICODE_CONTROL:
 		case G_UNICODE_FORMAT:
@@ -569,7 +591,7 @@ rb_search_fold (const char *original)
 			break;
 
 		case G_UNICODE_UNASSIGNED:
-			g_warning ("unassigned unicode character type found");
+			rb_debug ("unassigned unicode character type found");
 			/* fall through */
 
 		default:
@@ -858,3 +880,19 @@ rb_value_free (GValue *val)
 	g_free (val);
 }
 
+gboolean
+rb_str_in_strv (const char *needle, char **haystack)
+{
+	int i;
+
+	if (needle == NULL || haystack == NULL)
+		return FALSE;
+
+	for (i = 0; haystack[i] != NULL; i++) {
+		if (strcmp (needle, haystack[i]) == 0)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
