@@ -64,8 +64,6 @@
 #include "rb-file-helpers.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
-#include "rb-station-properties-dialog.h"
-#include "rb-new-station-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-shell-player.h"
@@ -148,7 +146,7 @@ GType rb_lastfm_src_get_type (void);
 static const char* const radio_options[][3] = {
 	{N_("Similar Artists radio"), "lastfm://artist/%s/similarartists", N_("Artists similar to %s")},
 	{N_("Tag radio"), "lastfm://globaltags/%s", N_("Tracks tagged with %s")},
-	{N_("Artist Fan radio"), "lastfm://artist/%s/fans", N_("Artists like by fans of %s")},
+	{N_("Artist Fan radio"), "lastfm://artist/%s/fans", N_("Artists liked by fans of %s")},
 	{N_("Group radio"), "lastfm://group/%s", N_("Tracks liked by the %s group")},
 	{NULL, NULL, NULL}
 };
@@ -1048,24 +1046,32 @@ rb_lastfm_source_title_from_uri (char *uri)
 		 *  will handle for us.
 		 */
 		if (data[4] != NULL) {
-			if (strcmp (data[4], "similarartists") == 0)
+			if (strcmp (data[4], "similarartists") == 0) {
 				title = g_strdup_printf (_("Artists similar to %s"), data[3]);
-			if (strcmp (data[4], "fans") == 0)
+			} if (strcmp (data[4], "fans") == 0) {
 				title = g_strdup_printf (_("Artists liked by fans of %s"), data[3]);
+			}
 		}
 
 	}
 
 	if (title == NULL && strcmp (data[2], "user") == 0) {
-		if (strcmp(data[4], "neighbours") == 0)
+		if (strcmp (data[4], "neighbours") == 0) {
 			title = g_strdup_printf (_("%s's Neighbour Radio"), data[3]);
-		if (strcmp(data[4], "recommended") == 0)
+		} else if (strcmp (data[4], "recommended") == 0) {
 			title = g_strdup_printf (_("%s's Recommended Radio: %s percent"), data[3], data[5]);
+		} else if (strcmp (data[4], "personal") == 0) {
+			title = g_strdup_printf (_("%s's Personal Radio"), data[3]);
+		}
 		/* subscriber? */
 	}
 
+	if (title == NULL && strcmp (data[2], "usertags") == 0) {
+		title = g_strdup_printf (_("%s's %s Radio"), data[3], data[4]);
+	}
+
 	if (title == NULL) {
-		title = g_strstrip (uri);
+		title = g_strdup (g_strstrip (uri));
 	}
 
 	g_strfreev (data);
@@ -1261,7 +1267,7 @@ impl_activate (RBSource *source)
 static gboolean
 impl_show_popup (RBSource *source)
 {
-	_rb_source_show_popup (source, "/LastfmSourcePopup");
+	/*_rb_source_show_popup (source, "/LastfmSourcePopup");*/
 	return TRUE;
 }
 
