@@ -46,7 +46,10 @@
 #include "rb-shell-player.h"
 #include "rb-cut-and-paste-code.h"
 
-static const GtkTargetEntry rb_entry_view_drag_types[] = {{  "text/uri-list", 0, 0 }};
+static const GtkTargetEntry rb_entry_view_drag_types[] = {
+	{ "application/x-rhythmbox-entry", 0, 0 },
+	{ "text/uri-list", 0, 1 }
+};
 
 struct RBEntryViewColumnSortData
 {
@@ -58,6 +61,7 @@ static void rb_entry_view_class_init (RBEntryViewClass *klass);
 static void rb_entry_view_init (RBEntryView *view);
 static GObject *rb_entry_view_constructor (GType type, guint n_construct_properties,
 					   GObjectConstructParam *construct_properties);
+static void rb_entry_view_dispose (GObject *object);
 static void rb_entry_view_finalize (GObject *object);
 static void rb_entry_view_set_property (GObject *object,
 				       guint prop_id,
@@ -221,6 +225,7 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+	object_class->dispose = rb_entry_view_dispose;
 	object_class->finalize = rb_entry_view_finalize;
 	object_class->constructor = rb_entry_view_constructor;
 
@@ -393,6 +398,61 @@ rb_entry_view_init (RBEntryView *view)
 	view->priv->column_key_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
+static void
+rb_entry_view_dispose (GObject *object)
+{
+	RBEntryView *view;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_ENTRY_VIEW (object));
+
+	view = RB_ENTRY_VIEW (object);
+
+	g_return_if_fail (view->priv != NULL);
+
+	if (view->priv->gconf_notification_id > 0) {
+		eel_gconf_notification_remove (view->priv->gconf_notification_id);
+		view->priv->gconf_notification_id = 0;
+	}
+
+	if (view->priv->sorting_gconf_notification_id > 0) {
+		eel_gconf_notification_remove (view->priv->sorting_gconf_notification_id);
+		view->priv->sorting_gconf_notification_id = 0;
+	}
+
+	if (view->priv->selection_changed_id > 0) {
+		g_source_remove (view->priv->selection_changed_id);
+		view->priv->selection_changed_id = 0;
+	}
+
+	if (view->priv->playing_pixbuf != NULL) {
+		g_object_unref (view->priv->playing_pixbuf);
+		view->priv->playing_pixbuf = NULL;
+	}
+
+	if (view->priv->paused_pixbuf != NULL) {
+		g_object_unref (view->priv->paused_pixbuf);
+		view->priv->paused_pixbuf = NULL;
+	}
+
+	if (view->priv->error_pixbuf != NULL) {
+		g_object_unref (view->priv->error_pixbuf);
+		view->priv->error_pixbuf = NULL;
+	}
+
+	if (view->priv->playing_model != NULL) {
+		g_object_unref (view->priv->playing_model);
+		view->priv->playing_model = NULL;
+	}
+
+	if (view->priv->model != NULL) {
+		g_object_unref (view->priv->model);
+		view->priv->model = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_entry_view_parent_class)->dispose (object);
+}
+
 static void
 rb_entry_view_finalize (GObject *object)
 {
@@ -405,32 +465,10 @@ rb_entry_view_finalize (GObject *object)
 
 	g_return_if_fail (view->priv != NULL);
 
-	if (view->priv->gconf_notification_id > 0)
-		eel_gconf_notification_remove (view->priv->gconf_notification_id);
-	if (view->priv->sorting_gconf_notification_id > 0)
-		eel_gconf_notification_remove (view->priv->sorting_gconf_notification_id);
-
-	if (view->priv->selection_changed_id > 0)
-		g_source_remove (view->priv->selection_changed_id);
-
 	g_hash_table_destroy (view->priv->propid_column_map);
 	g_hash_table_destroy (view->priv->column_sort_data_map);
 	g_hash_table_destroy (view->priv->column_key_map);
 
-	if (view->priv->playing_pixbuf != NULL)
-		g_object_unref (view->priv->playing_pixbuf);
-	if (view->priv->paused_pixbuf != NULL)
-		g_object_unref (view->priv->paused_pixbuf);
-	if (view->priv->error_pixbuf != NULL)
-		g_object_unref (view->priv->error_pixbuf);
-
-	if (view->priv->playing_model != NULL) {
-		g_object_unref (view->priv->playing_model);
-	}
-	if (view->priv->model != NULL) {
-		g_object_unref (view->priv->model);
-	}
-
 	g_free (view->priv->sorting_key);
 	g_free (view->priv->sorting_column_name);
 
@@ -492,13 +530,11 @@ rb_entry_view_set_model_internal (RBEntryView        *view,
 					 G_CALLBACK (rb_entry_view_rows_reordered_cb),
 					 view,
 					 0);
-	}
 
-	if (view->priv->sorting_column != NULL) {
-		rb_entry_view_resort_model (view);
-	}
+		if (view->priv->sorting_column != NULL) {
+			rb_entry_view_resort_model (view);
+		}
 
-	if (view->priv->model != NULL) {
 		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview),
 					 GTK_TREE_MODEL (view->priv->model));
 	}
@@ -1027,7 +1063,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data->propid = propid;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_long_cell_data_func;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_track_sort_func;
-		title = _("Trac_k");
+		title = _("Track");
 		key = "Track";
 		strings[0] = title;
 		strings[1] = "9999";
@@ -1038,7 +1074,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_TITLE_SORT_KEY;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_string_sort_func;
-		title = _("_Title");
+		title = _("Title");
 		key = "Title";
 		ellipsize = TRUE;
 		break;
@@ -1048,7 +1084,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_ARTIST_SORT_KEY;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_artist_sort_func;
-		title = _("Art_ist");
+		title = _("Artist");
 		key = "Artist";
 		ellipsize = TRUE;
 		break;
@@ -1058,7 +1094,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_ALBUM_SORT_KEY;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_album_sort_func;
-		title = _("_Album");
+		title = _("Album");
 		key = "Album";
 		ellipsize = TRUE;
 		break;
@@ -1068,7 +1104,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_GENRE_SORT_KEY;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_genre_sort_func;
-		title = _("_Genre");
+		title = _("Genre");
 		key = "Genre";
 		ellipsize = TRUE;
 		break;
@@ -1078,7 +1114,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_duration_cell_data_func;
 		sort_propid = cell_data->propid;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_ulong_sort_func;
-		title = _("Tim_e");
+		title = _("Time");
 		key = "Time";
 		strings[0] = title;
 		strings[1] = "000:00";
@@ -1090,7 +1126,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_year_cell_data_func;
 		sort_propid = cell_data->propid;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_date_sort_func;
-		title = _("_Year");
+		title = _("Year");
 		key = "Year";
 		strings[0] = title;
 		strings[1] = "0000";
@@ -1102,7 +1138,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_quality_cell_data_func;
 		sort_propid = cell_data->propid;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_ulong_sort_func;
-		title = _("_Quality");
+		title = _("Quality");
 		key = "Quality";
 		strings[0] = title;
 		strings[1] = _("000 kbps");
@@ -1115,7 +1151,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &column_width, NULL);
 		column_width = column_width * 5 + 5;
 		resizable = FALSE;
-		title = _("_Rating");
+		title = _("Rating");
 		key = "Rating";
 
 		renderer = rb_cell_renderer_rating_new ();
@@ -1137,7 +1173,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_play_count_cell_data_func;
 		sort_propid = cell_data->propid;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_ulong_sort_func;
-		title = _("_Play Count");
+		title = _("Play Count");
 		key = "PlayCount";
 		strings[0] = title;
 		strings[1] = _("Never");
@@ -1149,7 +1185,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_LAST_PLAYED;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_ulong_sort_func;
-		title = _("_Last Played");
+		title = _("Last Played");
 		key = "LastPlayed";
 		strings[0] = title;
 		strings[1] = rb_entry_view_get_time_date_column_sample ();
@@ -1161,7 +1197,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_FIRST_SEEN;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_ulong_sort_func;
-		title = _("_Date Added");
+		title = _("Date Added");
 		key = "FirstSeen";
 		strings[0] = title;
 		strings[1] = rb_entry_view_get_time_date_column_sample ();
@@ -1172,7 +1208,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_LAST_SEEN;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_ulong_sort_func;
-		title = _("Last _Seen");
+		title = _("Last Seen");
 		key = "LastSeen";
 		strings[0] = title;
 		strings[1] = rb_entry_view_get_time_date_column_sample ();
@@ -1183,7 +1219,7 @@ rb_entry_view_append_column (RBEntryView *view,
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_location_cell_data_func;
 		sort_propid = RHYTHMDB_PROP_LOCATION;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_location_sort_func;
-		title = _("L_ocation");
+		title = _("Location");
 		key = "Location";
 		ellipsize = TRUE;
 		break;
@@ -2036,16 +2072,14 @@ rb_entry_view_get_time_date_column_sample ()
 	 */
 
 	if (sample == NULL) {
-				/*    s  m  h   d  M    Y dw   dY  x */
-		struct tm someday = { 0, 0, 0, 30, 9, 100, 6, 274, 0};
+ 		time_t then;
+
+ 		/* A reasonable estimate of the widest friendly date
+ 		   is "Yesterday NN:NN PM" */
+ 		then = time (NULL) - 86400;
+ 		sample = rb_utf_friendly_time (then);
+  	}
 
-		/* Translators:  Please keep the translated date format
-		 * compact, and avoid variable-width items such as month and
-		 * day names wherever possible.  This allows us to disable
-		 * column autosizing, which makes the Rhythmbox UI much faster.
-		 */
-		sample = eel_strdup_strftime (_("%Y-%m-%d %H:%M"), &someday);
-	}
 	return sample;
 }
 
